#ifndef SEGMENTOH
#define SEGMENTOH

#include <cmath>
#include <iostream>
#include <vector>

#include "matriz.h"
#include "definicao.h"
#include "camada.h"
#include "celula.h"
#include "listacelula.h"
#include "face.h"
#include "listaface.h"
#include "ponto.h"
#include "transportesangue.h"
#include "compartimento.h"
#include "ambiente.h"
#include "boost/thread/thread.hpp"
#include <boost/thread/barrier.hpp>

using namespace std;

/*************************************************************************//**
* \class    Segmento 
* \brief    Definicao de um segmento.
* \author   Cyro Albuquerque Neto
* \date     Criacao: 28/08/2008
* \date     Revisao: 28/08/2008
*
* O segmento pode ter sistema de coordenadas cartesiano, 
* cilindrico ou esferico.
****************************************************************************/

class Segmento {

public:
    /**
     * Construtor
     * \param vDim Dimensao (1, 2 ou 3)
     * \param vSistCoord Sistema de coordenadas 
     * (CARTESIANO, CILINDRICO ou ESFERICO)
     * \param listaCamada lista das camadas em cada direcao
     * \param propTermica matriz com ponteiros das 
     * propriedades termicas
     * Se for cartesiano na ordem W,E,S,N,F,B. Se cilindrico
     * W,E,F,B (W adiatica caso 1D). Se esferica W,E.
     */
    Segmento() {contIt = 0; contador = 0; areaTotal = 0.0;}

    /**
    * Destrutor
    * \brief Apaga lista de celulas e faces
    */
    ~Segmento() {}

    bool getAtingiuPrecisaoCalcTemp() { return atingiuPrecisaoCalcTemp; }

    void setAtingiuPrecisaoCalcTemp(bool atingiuPrecisao){
    	atingiuPrecisaoCalcTemp = atingiuPrecisao;
    }

    bool getAtingiuPrecisaoCalcPressao() { return atingiuPrecisaoCalcPressao; }

    void setAtingiuPrecisaoCalcPressao(bool atingiuPrecisao){
    	atingiuPrecisaoCalcPressao = atingiuPrecisao;
    }

    void SetForma(IntPos dimensao, SistCoord sistCoordenada); // vai sair

    void SetGeometria(GeometriaSegmento vGeometria) {
        geometria = vGeometria;
    }
    GeometriaSegmento Geometria() const { return geometria; }

    void SetTamanhoExterno(double v1, double v2, double v3) {
        tamanhoExterno.SetPonto(v1,v2,v3);
        if (sistCoord==CARTESIANO)
            areaSuperficial=2.0*(v2*v3+v3*v1); //caso 2d
        else areaSuperficial=2.0*M_PI*v1*v3; //sem as tampas do cilindro
    }

    void Inicializa(vector<ListaCamada>& listaCamada,
            Matriz<CompTecido*>& compTec);

    double GerCalor();
    double VolumeTecido(TipoTecido tipo);
    double TempCompTecido(TipoTecido tipo);

    // Vazao de Sangue
    void SetVazSangueAnt(double numero) { vazSangueAnt = numero; }
    double VazSangueAnt() const { return vazSangueAnt; }
    void SetVazSanguePos(double numero) { vazSanguePos = numero; }
    double VazSanguePos() const { return vazSanguePos; }
    double VazSanguePeqVas();

    // Temperatura do sangue nas entrada do segmento
    double TempArtEnt() const { return tempArtEnt; }
    void SetTempArtEnt(double numero) { tempArtEnt = numero; }
    double TempVenEnt() const { return tempVenEnt; }
    void SetTempVenEnt(double numero) { tempVenEnt = numero; }

    // Concentracao do sangue nas entrada do segmento
    Gas<double> ConcArtEnt() const { return concArtEnt; }
    void SetConcArtEnt(Gas<double> numero) { concArtEnt = numero; }
    Gas<double> ConcVenEnt() const { return concVenEnt; }
    void SetConcVenEnt(Gas<double> numero) { concVenEnt = numero; }

    // Temperatura do sangue nos compartimentos arterial e venoso
    double TempArterial() const { return compArterial.Temperatura(); }
    double TempArterialAnt() const { return compArterial.TemperaturaAnt(); }
    double TempVenoso() const { return compVenoso.Temperatura(); }
    double TempVenosoAnt() const { return compVenoso.TemperaturaAnt(); }

    // Concentracao do sangue nos compartimentos arterial e venoso
    Gas<double> ConcArterial() const { return compArterial.Concentracao(); }
    double ConcArterial(IntPos i) const { return compArterial.Concentracao(i); }
    Gas<double> ConcVenoso() const { return compVenoso.Concentracao(); }
    double ConcVenoso(IntPos i) const { return compVenoso.Concentracao(i); }

    // Coeficiente de conveccao
    void SetResistTemperatura(double numero) { resistTemperatura = numero; }
    void SetResistEvaporacao(double numero) { resistEvaporacao = numero; }
    void SetSupMolhada(double numero) { supMolhada = numero; }
    double SupMolhada() const { return supMolhada; }

    // Coeficiente de transferencia de calor entre compartimento arterial e venoso
    void SetCoefArtVen(double valor) { coefArtVen=valor; }

    // Volume dos compartimentos arterial e venoso
    void SetVolArtVen(double vArt, double vVen) {
            compArterial.SetVolume(vArt);
            compVenoso.SetVolume(vVen);
        }
    double VolCompArterial() { return compArterial.Volume(); }
    double VolCompVenoso() { return compVenoso.Volume(); }


    //Compartimento tecidual
    CompTecido* PtCompTecido(IntPos numero) { return &lCompTecido[numero]; }
    //CompTecido* PtUltimoCompTecido() { return &lCompTecido.back(); }
    IntPos QuantCompTecido() { return lCompTecido.size(); }
    //vector<CompTecido>* PtListaCompTecido() { return &lCompTecido; }
    //TipoTecido TipoTecido(IntPos numComp) { return lCompTecido[numComp].TipoTec(); }

    void NovoCompTecido(PropTecido * pTec, TipoTecido tipo) { 
        CompTecido comp;
        lCompTecido.push_back(comp);
        lCompTecido.back().SetPropTecido(pTec,tipo);
    }

    //Se for tronco
    bool EhTronco() { return ehTronco; }
    void SetEhTronco(bool valor) { ehTronco=valor; }

    //Se tecido � pulm�o
    bool EhPulmao(IntPos numComp) {
        if (lCompTecido[numComp].NomeTecido()==PULMAO) return true;
        else return false;
    }

    //Talvez seja usado para ordem das coordenadas, ver variavel.
    //void SetCoord(Coordenada v1, Coordenada v2, Coordenada v3)
    //  { coordenadas.SetPonto(v1,v2,v3); } 
    
    void CalcTempCel();
    void PassoCalculoTemperatura();
    void PassoCalculoPressao();
    void PassoCalculo();

    void AtualizaValorAnt();

    Ponto<double> TamanhoExterno() const { return tamanhoExterno; }
    
    // Celulas
    Celula* Celulas(IntPos valor) { return lCelula.Em(valor); }
    Celula* Celulas(IntPos val1, IntPos val2) 
        { return lCelula.Em(val1,val2); }
    Celula* Celulas(IntPos val1, IntPos val2, IntPos val3) 
        { return lCelula.Em(val1,val2,val3); }
    Matriz<Celula*>* CelulaVet() { return lCelula.Vet(); }
    IntP CompTecDaCel(IntP numero) { return lCelula.Em(numero)->NumCompCel(); } 

    // Faces
    Face* Faces(IntPos valor) { return lFace.Em(valor); }
    vector<Face*>* FaceVet() { return lFace.Vet(); }

    //Calor perdido pro ambiente
    double CalorProAmbiente() { return calorProAmbiente; }
    
    double CalorRespiracao(double tempAr, double uRel);

    //double ConsumoO2();
    //double GerCalor();
    //double VazSangueSemMusculo();
    //double VolMusculo();

    void SetAmbiente(Ambiente* valor) { amb = valor;}

    double AreaSupCelulas();

    void AjustaSupMolhada(double tPeleMedia, double tInterna);

    static double ventilacao; // [m3/s]

    void AjustaFracaoInspirada(double fracaoO2, double fracaoCO2) {
        presInspirada(0) = fracaoO2*(amb->PressBar() - 47.0*133.32237);
        presInspirada(1) = fracaoCO2*(amb->PressBar() - 47.0*133.32237);
        //usar ao inves de 47.0 pressao de vapor d'agua calculada
    }
    
    void SaidaDadosArquivo(ostream &arquivo);

    void setNome(string n){ nome = n; }
    string getNome(){ return nome; }
    void AtualizaEntreCalculos();
    //bool EhEspelhado() const{return ehEspelhado;};
    //void setEhEspelhado(bool esp){ehEspelhado = esp;}
    double totTempoCTC;
    double totTempoAM;
    double totTempoCTV;
    double totTempoCTA;
    double totTempoCTCT;
    double totTempoCorpoCTC;
    double totTtempoACV;
    unsigned long contIt;
    unsigned long contador;

private:

    //   presCapPulm, presAlveolar;

    /// Numero de dimensoes (1, 2 ou 3)
    IntPos nDim;

    /// Sistema de coordenada (CARTESIANO, CILINDRICO, CIL_RZ)
    SistCoord sistCoord;

    /// Geometria do segmento (RADIAL,AXIAL,POLAR,RETANGULAR)
    GeometriaSegmento geometria;

    /// Tamanho externo do segmento
    Ponto<double> tamanhoExterno;

    /// Area Superficial
    double areaSuperficial;

    /// Lista de celulas 
    ListaCelula lCelula;

    /// Lista de faces
    ListaFace lFace;
    
    // Propriedades do sangue
    static double const densidadeSangue = 1059.0;
    static double const calorEspSangue = 3850.0;

    // Coeficiente de solubilidade dos tecidos
    //static double const coefSolTecO2 = 2.36e-7;
    //static double const coefSolMuscCO2 = 3.42e-5;
    //static double const coefSolTecCO2 = 2.38e-5;
    
    // Coeficiente de transferencia de calor entre compartimento arterial e venoso
    double coefArtVen;

    // Vazao de sangue
    double vazSangueAnt;
    double vazSanguePos;
    double vazSanguePeqVas;

    // Temperatura do sangue nas entradas do segmento
    double tempArtEnt;
    double tempVenEnt;

    /// CoefConveccao
    double resistTemperatura, resistEvaporacao;
    double supMolhada;

    /// Temperatura superficial m�dia
    double tempSuperficialMedia;

    // Calcula temperatura no proximo instante nos compartimentos
    double CalcTempArterial();
    double CalcTempVenoso();

    // Calculo press�es parciais
    void CalcPressaoTecido();
    Gas<double> CalcPressaoArterial();
    Gas<double> CalcPressaoVenoso();

    // Calculo press�o parcial nos tecidos
    Gas<double> CalcPressaoTecido(IntPos iTec);

    // Calcula press�o parcial dos gases no espa�o alveolar
    Gas<double> CalcPressaoAlveolar();

    // Calcula press�o parcial dos gases nos capilares pulmonares
    Gas<double> CalcPressaoCapPulm(IntPos iCap, IntPos iTec);

    // Ambiente
    static Ambiente *amb;

    // Press�o inspirada
    static Gas<double> presInspirada;

    //Calor perdido pro ambiente
    double calorProAmbiente;

    // Concentracao do sangue nas entradas do segmento
    Gas<double> concArtEnt;
    Gas<double> concVenEnt;

    /// Compartimentos de sangue
    CompSangue compArterial, compVenoso;
    
    /// Compartimentos de tecido
    vector<CompTecido> lCompTecido;

    /// Compartimento do pulmao
    static CompPulmao compPulmao;

    /// Se for tronco
    bool ehTronco;

    //void AtualizaTamanhoExterno(vector<ListaCamada>& lCamada);
    void CalculaTempCompTec();

    //void CalcCoefTermFace(Celula* cel, IntPos iF, Face* face);
    void AtualizaCoefCte();
    void AtualizaCoefVar();
    //void CalcConcPresCel();

    /** Calcula calor trocado com o ambiente pela pele
     * \param tSuperficial Temperatura superficial da pele [oC]
     * \return Calor trocado com o ambiente [W/m2]
     */
    double CalorAmbiente(double tSuperficial);
   
    double Sudorese(double tempPeleMedia, double tempPeleLocal, double tempInterna);

    bool atingiuPrecisaoCalcTemp;

    bool atingiuPrecisaoCalcPressao;

    string nome;

    void AtualizaCapilaresPulmonar(int i);

    void AtualizaCapilaresPulmonarSeq(int i);

    void AtualizaCapilaresPulmonarPar(vector<int> indiceCapilares, int i);

    bool ehEspelhado;

    // Funcoes de concorrencia
    void calcTempCelRet(int iniI, int limiteI, int iniJ, int limiteJ, boost::mutex& m, boost::barrier& b);

    void testectcr();

    void CalcTempCelAf(int iniI, int iniJ, int limI, int limJ, boost::mutex& m, boost::barrier& bHor, boost::barrier& bVert, boost::barrier& bProg);

    void CalcTempCelAfMid(int iniI, int iniJ, int limI, int limJ, boost::mutex& m, boost::barrier& bHor, boost::barrier& bVert, boost::barrier& bProg);

    void CalcTempCelSwt();

    double areaTotal;

};

#endif
